#include <stdio.h>
#include "geoipcity_amxx.h"

GeoIP *gi = NULL;

static cell AMX_NATIVE_CALL amx_geoip_city( AMX *amx, cell *params )
{
	int len = 0;
	char *ip = MF_GetAmxString( amx, params[1], 0, &len );

	StripPort( ip );
	GeoIPRecord* rec = GeoIP_record_by_addr( gi, ip );
	
	if ( rec )
	{
		cell res = MF_SetAmxString( amx, params[2], rec->city ? rec->city : "error", params[3] );
		free( rec );
		return res;
	}
	else
		return MF_SetAmxString( amx, params[2], "error", params[3] );
}

static cell AMX_NATIVE_CALL amx_geoip_latitude( AMX *amx, cell *params )
{
	int len = 0;
	char *ip = MF_GetAmxString( amx, params[1], 0, &len );

	StripPort( ip );
	GeoIPRecord* rec = GeoIP_record_by_addr( gi, ip );
	
	return ( rec ) ? amx_ftoc( rec->latitude ) : 0;
}

static cell AMX_NATIVE_CALL amx_geoip_longitude( AMX *amx, cell *params )
{
	int len = 0;
	char *ip = MF_GetAmxString( amx, params[1], 0, &len );

	StripPort(ip);
	GeoIPRecord* rec = GeoIP_record_by_addr( gi, ip );
	
	return ( rec ) ? amx_ftoc( rec->longitude ) : 0;
}

void OnAmxxAttach()
{
	char *path = MF_BuildPathname( "%s/GeoIPCity.dat", MF_GetLocalInfo( "amxx_datadir", "addons/amxmodx/data" ) );
	gi = GeoIP_open( path, GEOIP_MEMORY_CACHE );

	if (gi == NULL) 
	{
		MF_Log( "Failed to instantiate GeoIPCity!" );
		return;
	}

	char *db_info = GeoIP_database_info( gi );

	MF_Log( "Database info: %s", db_info );
	MF_AddNatives( geoip_natives );
}

void OnAmxxDetach()
{
	GeoIP_delete( gi );
	gi = NULL;
}

AMX_NATIVE_INFO geoip_natives[] = 
{
	{ "geoip_city"		,	amx_geoip_city		},
	{ "geoip_latitude"	,	amx_geoip_latitude	},
	{ "geoip_longitude"	,	amx_geoip_longitude	},
	{ NULL				,	NULL				},
};
